const console = require('../utils/logger.js');

const hostToPathMiddleware = (req, res, next) => {
    console.debug(`Entering host-to-path middleware (x-correlationid: ${req.headers['x-correlationid']})`);
    const host = (
        req.headers['x-forwarded-host'] 
        ?? req.headers[':authority'] 
        ?? req.headers.host
        ?? 'localhost'
    ).split(':')[0];
    if (host.includes('.') && host.startsWith('app-')) {
        const hostWithoutDomain = host.split('.')[0];
        const [name, version] = hostWithoutDomain.slice(4).split('-port')[0].split('-v');
        if (name && version) {
            const path = name + '-' + (version.replaceAll('_', '.'));
            console.debug(
                `Rewriting host '${hostWithoutDomain}' to path '${path}' `+
                `(x-correlationid: ${req.headers['x-correlationid']})`
            );
            if (req.url === '/') {
                // Fix AppRouter welcome file logic
                req.headers['x-forwarded-path'] = '/';
            }
            req.url = '/' + path + req.url;
            console.debug(
                `New path is '${req.url}'`+
                `(x-correlationid: ${req.headers['x-correlationid']})`
            );
            next();
            return;
        }
    }
    console.debug(
        `The host '${host}' does not include subdomain part or subdomain does not start with 'app-'`+
        `(x-correlationid: ${req.headers['x-correlationid']})`
    );
    next();
};

module.exports = hostToPathMiddleware;
