const console = require('../utils/logger.js');
const { serviceInstanceDestination } = require('../utils/configuration.js');

if (serviceInstanceDestination) {
    const serviceInstanceDestinationMiddleware = (req, res, next) => {
        console.debug(
            'Entering service-instance-destination middleware'+
            `(x-correlationid: ${req.headers['x-correlationid']})`
        );
        if (req && req.internalUrl && req.internalUrl.destination && req.internalUrl.destination.url) {
            const url = new URL(req.internalUrl.destination.url);
            const auth = url.password ?
                Buffer.from(url.username + ':' + url.password).toString('base64') :
                url.username;
            req.headers['bas-destination-instance-cred'] = auth;
            console.debug(
                'Adding \'bas-destination-instance-cred\' header to request '+
                `(x-correlationid: ${req.headers['x-correlationid']})`,
                auth
            );
        }
        next();
    };
    module.exports = serviceInstanceDestinationMiddleware;
} else {
    const nopServiceInstanceDestinationMiddleware = (req, res, next) => next();
    module.exports = nopServiceInstanceDestinationMiddleware;
}
