const console = require('../../utils/logger.js');
const OAUTH_TOKEN = /\/oauth\/token.*/;
const TOKEN = JSON.stringify({
    'access_token': 'MOCK.ACCESS.TOKEN',
    'token_type': 'bearer',
    'expires_in': 43199,
    'scope': 'uaa.resource',
    'jti': '915f7795bf2f4ea5a77b138dc8a50709'
});

const xsuaaMiddleware = (req, res, next) => {
    if (req.method === 'POST' && req.url.match(OAUTH_TOKEN)) {
        console.debug(`Returning JWT (x-correlationid: ${req.headers['x-correlationid']})`, TOKEN);
        res.setHeader('Content-Type', 'application/json');
        return res.end(TOKEN);
    }
    return next();
};

module.exports = () => {
    const { login } = require('../../utils/configuration.js');

    if (login) {
        const { add, getSapCloudService } = require('../../utils/vcap.js');
        const binding = getSapCloudService(login);
        if (binding && binding.credentials && binding.credentials.uaa) {
            console.debug('Adding business service credentials as XSUAA service for login');
            add({
                service: 'xsuaa',
                plan: 'application',
                credentials: binding.credentials.uaa,
                tags: ['xsuaa']
            });
        } else {
            console.error(`SAP Cloud Service '${login}' is not bound`);
            throw new Error(`SAP Cloud Service '${login}' is not bound`);
        }
    }

    return xsuaaMiddleware;
};
